/************************************************
*	monitor_house.cpp	*
*						*
*	This program sends and receives 	*
*	data to/from a usb serial port		*
*						*
*	Compile: make				*
*						*
*		J. Savage			*
*		M. Matamoros			*
*		UNAM				*
*						*
*						*
*			8/22/14			*
*************************************************/

// Contant definitions
#define PORT 2025 // change the value of PORT accordingly
#define MAX_NUM_CHARACTERS 1000


#include <stdio.h>

#include <pthread.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>


// Include standard libraries
#include <cstdlib>
#include <iostream>
#include <string>
#include "Module.h"
// Class that manages shared variable updates
#include <uRobotics/api/SharedVariableSubscriptionReport.h>
// Wrappers for shared variables
#include <uRobotics/api/psv/StringSharedVariable.h>
#include <unistd.h>


// global variables
int display;
long frecuency;
// A wrapper for a shared variable that handles the arduino input (data to)
StringSharedVariable *arduino_in = NULL;
// A wrapper for a shared variable that handles the arduino output (data from)
StringSharedVariable *arduino_out = NULL;

// Include Blacboard libraries
#include "../sensor_actuator_src/setup_BB_C++.h"

// It sends a command though the arduino_out variable
void send_command(char *line){
 	std::string s;

 	s = line;
        //std::cout << "Send command: " << s;
        arduino_out->tryWrite(s);

}


inline void wait_ms( unsigned long ms ) {
    usleep( ms * 1000 );
}

/**
* The function that will handle when a shared variable (arduino_out) is written
* @param report Contains the information of the shared variable which information has been written
*/

void arduino_in_written(const SharedVariableSubscriptionReport<std::string>& report){
        char line[MAX_NUM_CHARACTERS];
        int send_terminal(char *);

        strcpy(line, report.getValue().c_str());

        if(display == 1){
                wait_ms(frecuency);
                std::cout << "[SENSOR] " << line <<std::endl;
        }
}


void arduino_out_written(const SharedVariableSubscriptionReport<std::string>& report){
        char line[MAX_NUM_CHARACTERS];
        int send_terminal(char *);

        strcpy(line, report.getValue().c_str());

        if(display == 1){
                wait_ms(frecuency);
                std::cout << "[ACTUATOR] " << line <<std::endl;
        }
}



/* it gets line inputs */
int get_inputs(int argc, char **argv, int *disp, long *fr) {
	int i;
	
	/* It sets default values */
	*disp=1;
	*fr=0;
	
	/* reads input parameters */
	if (argc>1) {
		for(i=1;i<argc;i++){
			if (!strcmp(argv[i],"-h")){
				printf("Sintax: monitor_house -d display_data(1/0) -f mseconds(0 continuous) \n");
				printf("example:\nmonitor_house -d 1 -f 1000\n");
				printf("Default: monitor_house -d 1 -f 0\n");
				exit(1);
			}
			if (!strcmp(argv[i],"-f")){
				*fr=atoi(argv[i+1]);
			}
			if (!strcmp(argv[i],"-d")){
				*disp=atoi(argv[i+1]);
			}
		}
	}
	else{
		printf("It will use default values: serial_house -d %d -f %ld\n",*disp,*fr);
	}
	return(0);
}




/* Get a line from the keyword, and give its size */
int getline(char *s,int lim) {
        int c,i;

        for(i=0;i<lim-1 && (c=getchar() ) != EOF && c != '\n'; ++i)
                s[i]=c;
        s[i]='\0';
        return(i);
}


int main(int argc, char *argv[])
{
	char line[MAX_NUM_CHARACTERS],command[MAX_NUM_CHARACTERS];
	int disp;
	long fr;
	int ptr;
	Module *module;


	// Configure the module to run on PORT
	module=init_BB(PORT);

	/* it gets line inputs */
	get_inputs(argc,argv,&disp,&fr);

	display=disp;
	frecuency=fr;


	while(1){
		/* read data from the keyboard */
		printf("-> ");
		ptr=getline(line,400); 
		//strcat(line,"\n");
		send_command(line);
	}

}

