/************************************************
*	sensor_actuator.cpp	*
*						*
*	This program sends and receives 	*
*	data to/from a usb serial port		*
*	and from the Blackboard			*
*						*
*						*
*		J. Savage			*
*		M. Matamoros			*
*		UNAM				*
*						*
*						*
*			10/07/2014		*
*************************************************/

// Contant definitions
#define PORT 2011 // change the value of PORT accordingly
#define MAX_NUM_CHARACTERS 1000

// Include standard libraries
#include <stdio.h>
#include <time.h>
#include <pthread.h>
#include <string.h>
#include <stdlib.h>
#include <cstdlib>
#include <iostream>
#include "Module.h"
// Class that manages shared variable updates
#include <uRobotics/api/SharedVariableSubscriptionReport.h>
// Wrappers for shared variables
#include <uRobotics/api/psv/StringSharedVariable.h>

// global variables
int fd1,fd2;
int display=1;
long frecuency=0;
// A wrapper for a shared variable that handles the arduino input (data to)
StringSharedVariable *arduino_in = NULL;
// A wrapper for a shared variable that handles the arduino output (data from)
StringSharedVariable *arduino_out = NULL;

// Include Blacboard libraries
#include "setup_BB_C++.h"


// Function to wait milliseconds
inline void wait_ms( unsigned long ms ) {
    usleep( ms * 1000 );
}



// This function handles when the shared variable sensor (arduino_in) is written
void arduino_in_written(const SharedVariableSubscriptionReport<std::string>& report){
        char line[MAX_NUM_CHARACTERS];

        strcpy(line, report.getValue().c_str());

        if(display == 1){
                std::cout << "SENSOR " << line <<std::endl;
                wait_ms(frecuency);
        }
}



// This function handles when the shared variable actuator (arduino_out) is written
void arduino_out_written(const SharedVariableSubscriptionReport<std::string>& report){
        char line[MAX_NUM_CHARACTERS];
        int send_terminal(char *);

        strcpy(line, report.getValue().c_str());

        if(display == 1){
                std::cout << "[ACTUATOR] " << line <<std::endl;
                //wait_ms(frecuency);
        }

	// it sends the data to the serial port 
        send_terminal(line);

}


char* myasctime(const struct tm *timeptr)
{
  static const char wday_name[][4] = {
    "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
  };
  static const char mon_name[][4] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
  };
  static char result[26];
  sprintf(result, "%.3s %.3s%3d %.2d:%.2d:%.2d %d",
    wday_name[timeptr->tm_wday],
    mon_name[timeptr->tm_mon],
    timeptr->tm_mday, timeptr->tm_hour,
    timeptr->tm_min, timeptr->tm_sec,
    1900 + timeptr->tm_year);
  return result;
}



// This function reads from usb serial port
void *read_serial_port(void *arg) {
        time_t rawtime;
	struct tm * timeinfo;
        char c;
        std::string s;

        while (1) {       // loop for input 
                read(fd2, &c, (size_t)1);
                if(c == '\r') continue;
                if(c != '\n'){
                        s.append(1, c);
                        continue;
                }
                time (&rawtime);
  		timeinfo = localtime (&rawtime);
                s = s + " " + myasctime(timeinfo);
                //std::cout << s <<std::endl ;
                arduino_in->tryWrite(s);
                s.clear();
		// Wait one second
                //wait_ms(1000);
        }
}



/* it gets line inputs */
int get_inputs(int argc, char **argv, char *port, int *baud) {
	int i;
	
	/* It sets default values */
	strcpy(port,"/dev/ttyACM0");
	*baud=9600;
	
	/* reads input parameters */
	if (argc>1) {
		for(i=1;i<argc;i++){
			if (!strcmp(argv[i],"-h")){
				printf("Sintax: serial_house -p port -b baudage \n");
				printf("example:\nserial_house -p /dev/ttyUSB0 -b 9600\n");
				printf("Default: serial_house-p  /dev/ttyACM0 -b 9600\n");
				exit(1);
			}
			if (!strcmp(argv[i],"-p")){
				strcpy(port,argv[i+1]);
			}
			if (!strcmp(argv[i],"-b")){
				*baud=atoi(argv[i+1]);
			}
		}
	}


	printf("sensor_actuator -p %s -b %d\n",port,*baud);
	return(0);
}



int main(int argc, char *argv[])
{
	char s[MAX_NUM_CHARACTERS],c;
	char line[MAX_NUM_CHARACTERS],command[MAX_NUM_CHARACTERS];
	int ptr,flag,baud;
	pthread_t a_thread;
	char serial_port[32];
	int open_terminal(char *,int);
	int getline(char *,int);
	int send_terminal(char *);
	int close_terminal(void);
	Module *module;


	// it gets line inputs
	get_inputs(argc,argv,serial_port,&baud);


        // Configure the Blackboard module to run on PORT
        module=init_BB_sensor_actuator(PORT);
	printf("Blackboard sensor_actuator module to run on PORT %d\n",PORT);

	/* it opens the serial port */
	if( (fd2=open_terminal(serial_port,baud) ) == 0) {
		printf("Device not connected\n");
		exit(1);
	}

	// It sets a thread for reading the input serial values
	pthread_create(&a_thread, NULL, read_serial_port, NULL);


	while(1){
		wait_ms(1);
	}

	close_terminal();

}




